@@ -24,18 +24,21 @@
 
 #include "pango-glyph-item.h"
 #include "pango-impl-utils.h"
+#include "pango-attributes-private.h"
 
 #define LTR(glyph_item) (((glyph_item)->item->analysis.level % 2) == 0)
 
 /**
  * pango_glyph_item_split:
- * @orig: a #PangoItem
+ * @orig: a `PangoItem`
  * @text: text to which positions in @orig apply
  * @split_index: byte index of position to split item, relative to the start of the item
  *
  * Modifies @orig to cover only the text after @split_index, and
  * returns a new item that covers the text before @split_index that
- * used to be in @orig. You can think of @split_index as the length of
+ * used to be in @orig.
+ *
+ * You can think of @split_index as the length of
  * the returned item. @split_index may not be 0, and it may not be
  * greater than or equal to the length of @orig (that is, there must
  * be at least one byte assigned to each item, you can't create a
@@ -131,11 +134,11 @@ pango_glyph_item_split (PangoGlyphItem *orig,
 
 /**
  * pango_glyph_item_copy:
- * @orig: (nullable): a #PangoGlyphItem, may be %NULL
+ * @orig: (nullable): a `PangoGlyphItem`, may be %NULL
  *
- * Make a deep copy of an existing #PangoGlyphItem structure.
+ * Make a deep copy of an existing `PangoGlyphItem` structure.
  *
- * Return value: (nullable): the newly allocated #PangoGlyphItem, which should
+ * Return value: (nullable): the newly allocated `PangoGlyphItem`, which should
  *               be freed with pango_glyph_item_free(), or %NULL
  *               if @orig was %NULL.
  *
@@ -159,9 +162,9 @@ pango_glyph_item_copy  (PangoGlyphItem *orig)
 
 /**
  * pango_glyph_item_free:
- * @glyph_item: (nullable): a #PangoGlyphItem, may be %NULL
+ * @glyph_item: (nullable): a `PangoGlyphItem`, may be %NULL
  *
- * Frees a #PangoGlyphItem and resources to which it points.
+ * Frees a `PangoGlyphItem` and resources to which it points.
  *
  * Since: 1.6
  **/
@@ -186,13 +189,13 @@ G_DEFINE_BOXED_TYPE (PangoGlyphItem, pango_glyph_item,
 
 /**
  * pango_glyph_item_iter_copy:
- * @orig: (nullable): a #PangoGlyphItemIter, may be %NULL
+ * @orig: (nullable): a `PangoGlyphItem`Iter, may be %NULL
  *
- * Make a shallow copy of an existing #PangoGlyphItemIter structure.
+ * Make a shallow copy of an existing `PangoGlyphItemIter` structure.
  *
- * Return value: (nullable): the newly allocated #PangoGlyphItemIter, which should
- *               be freed with pango_glyph_item_iter_free(), or %NULL
- *               if @orig was %NULL.
+ * Return value: (nullable): the newly allocated `PangoGlyphItemIter`,
+ *   which should be freed with pango_glyph_item_iter_free(), or %NULL
+ *   if @orig was %NULL.
  *
  * Since: 1.22
  **/
@@ -213,9 +216,9 @@ pango_glyph_item_iter_copy  (PangoGlyphItemIter *orig)
 
 /**
  * pango_glyph_item_iter_free:
- * @iter: (nullable): a #PangoGlyphItemIter, may be %NULL
+ * @iter: (nullable): a `PangoGlyphItemIter`, may be %NULL
  *
- * Frees a #PangoGlyphItemIter created by pango_glyph_item_iter_copy().
+ * Frees a `PangoGlyphItem`Iter.
  *
  * Since: 1.22
  **/
@@ -234,13 +237,14 @@ G_DEFINE_BOXED_TYPE (PangoGlyphItemIter, pango_glyph_item_iter,
 
 /**
  * pango_glyph_item_iter_next_cluster:
- * @iter: a #PangoGlyphItemIter
+ * @iter: a `PangoGlyphItemIter`
  *
  * Advances the iterator to the next cluster in the glyph item.
- * See #PangoGlyphItemIter for details of cluster orders.
  *
- * Return value: %TRUE if the iterator was advanced, %FALSE if we were already on the
- *  last cluster.
+ * See `PangoGlyphItemIter` for details of cluster orders.
+ *
+ * Return value: %TRUE if the iterator was advanced,
+ *   %FALSE if we were already on the  last cluster.
  *
  * Since: 1.22
  **/
@@ -281,7 +285,7 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (glyphs->log_clusters[glyph_index] > cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
@@ -304,7 +308,7 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
 	      break;
 	    }
 
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (glyphs->log_clusters[glyph_index] > cluster)
 	    {
 	      iter->end_index = item->offset + glyphs->log_clusters[glyph_index];
 	      iter->end_char += pango_utf8_strlen (iter->text + iter->start_index,
@@ -315,18 +319,22 @@ pango_glyph_item_iter_next_cluster (PangoGlyphItemIter *iter)
     }
 
   iter->end_glyph = glyph_index;
+
+  g_assert (iter->start_char <= iter->end_char);
+  g_assert (iter->end_char <= item->num_chars);
+
   return TRUE;
 }
 
 /**
  * pango_glyph_item_iter_prev_cluster:
- * @iter: a #PangoGlyphItemIter
+ * @iter: a `PangoGlyphItemIter`
  *
  * Moves the iterator to the preceding cluster in the glyph item.
- * See #PangoGlyphItemIter for details of cluster orders.
+ * See `PangoGlyphItemIter` for details of cluster orders.
  *
- * Return value: %TRUE if the iterator was moved, %FALSE if we were already on the
- *  first cluster.
+ * Return value: %TRUE if the iterator was moved,
+ *   %FALSE if we were already on the first cluster.
  *
  * Since: 1.22
  **/
@@ -368,7 +376,7 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 
 	  glyph_index--;
 
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (glyphs->log_clusters[glyph_index] < cluster)
 	    {
 	      glyph_index++;
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
@@ -392,7 +400,7 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
 
 	  glyph_index++;
 
-	  if (glyphs->log_clusters[glyph_index] != cluster)
+	  if (glyphs->log_clusters[glyph_index] < cluster)
 	    {
 	      glyph_index--;
 	      iter->start_index = item->offset + glyphs->log_clusters[glyph_index];
@@ -404,18 +412,23 @@ pango_glyph_item_iter_prev_cluster (PangoGlyphItemIter *iter)
     }
 
   iter->start_glyph = glyph_index;
+
+  g_assert (iter->start_char <= iter->end_char);
+  g_assert (0 <= iter->start_char);
+
   return TRUE;
 }
 
 /**
  * pango_glyph_item_iter_init_start:
- * @iter: a #PangoGlyphItemIter
+ * @iter: a `PangoGlyphItemIter`
  * @glyph_item: the glyph item to iterate over
  * @text: text corresponding to the glyph item
  *
- * Initializes a #PangoGlyphItemIter structure to point to the
+ * Initializes a `PangoGlyphItemIter` structure to point to the
  * first cluster in a glyph item.
- * See #PangoGlyphItemIter for details of cluster orders.
+ *
+ * See `PangoGlyphItemIter` for details of cluster orders.
  *
  * Return value: %FALSE if there are no clusters in the glyph item
  *
@@ -447,13 +460,14 @@ pango_glyph_item_iter_init_start (PangoGlyphItemIter  *iter,
 
 /**
  * pango_glyph_item_iter_init_end:
- * @iter: a #PangoGlyphItemIter
+ * @iter: a `PangoGlyphItemIter`
  * @glyph_item: the glyph item to iterate over
  * @text: text corresponding to the glyph item
  *
- * Initializes a #PangoGlyphItemIter structure to point to the
+ * Initializes a `PangoGlyphItemIter` structure to point to the
  * last cluster in a glyph item.
- * See #PangoGlyphItemIter for details of cluster orders.
+ *
+ * See `PangoGlyphItemIter` for details of cluster orders.
  *
  * Return value: %FALSE if there are no clusters in the glyph item
  *
@@ -545,12 +559,14 @@ split_before_cluster_start (ApplyAttrsState *state)
  * pango_glyph_item_apply_attrs:
  * @glyph_item: a shaped item
  * @text: text that @list applies to
- * @list: a #PangoAttrList
+ * @list: a `PangoAttrList`
  *
- * Splits a shaped item (PangoGlyphItem) into multiple items based
- * on an attribute list. The idea is that if you have attributes
+ * Splits a shaped item (`PangoGlyphItem`) into multiple items based
+ * on an attribute list.
+ *
+ * The idea is that if you have attributes
  * that don't affect shaping, such as color or underline, to avoid
- * affecting shaping, you filter them out (pango_attr_list_filter()),
+ * affecting shaping, you filter them out ([method@Pango.AttrList.filter]),
  * apply the shaping process and then reapply them to the result using
  * this function.
  *
@@ -558,7 +574,7 @@ split_before_cluster_start (ApplyAttrsState *state)
  * to that cluster; for instance, if half of a cluster is underlined
  * and the other-half strikethrough, then the cluster will end
  * up with both underline and strikethrough attributes. In these
- * cases, it may happen that item->extra_attrs for some of the
+ * cases, it may happen that @item->extra_attrs for some of the
  * result items can have multiple attributes of the same type.
  *
  * This function takes ownership of @glyph_item; it will be reused
@@ -566,7 +582,7 @@ split_before_cluster_start (ApplyAttrsState *state)
  *
  * Return value: (transfer full) (element-type Pango.GlyphItem): a
  *   list of glyph items resulting from splitting @glyph_item. Free
- *   the elements using pango_glyph_item_free(), the list using
+ *   the elements using [method@Pango.GlyphItem.free], the list using
  *   g_slist_free().
  *
  * Since: 1.2
@@ -576,7 +592,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 			      const char       *text,
 			      PangoAttrList    *list)
 {
-  PangoAttrIterator *iter = pango_attr_list_get_iterator (list);
+  PangoAttrIterator iter;
   GSList *result = NULL;
   ApplyAttrsState state;
   gboolean start_new_segment = FALSE;
@@ -599,15 +615,16 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 
   /* Advance the attr iterator to the start of the item
    */
+  _pango_attr_list_get_iterator (list, &iter);
   do
     {
-      pango_attr_iterator_range (iter, &range_start, &range_end);
+      pango_attr_iterator_range (&iter, &range_start, &range_end);
       if (range_end > glyph_item->item->offset)
 	break;
     }
-  while (pango_attr_iterator_next (iter));
+  while (pango_attr_iterator_next (&iter));
 
-  state.segment_attrs = pango_attr_iterator_get_attrs (iter);
+  state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
 
   is_ellipsis = (glyph_item->item->analysis.flags & PANGO_ANALYSIS_FLAG_IS_ELLIPSIS) != 0;
 
@@ -636,7 +653,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	{
 	  result = g_slist_prepend (result,
 				    split_before_cluster_start (&state));
-	  state.segment_attrs = pango_attr_iterator_get_attrs (iter);
+	  state.segment_attrs = pango_attr_iterator_get_attrs (&iter);
 	}
 
       start_new_segment = FALSE;
@@ -655,8 +672,8 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	   */
 	  start_new_segment = TRUE;
 
-	  have_next = pango_attr_iterator_next (iter);
-	  pango_attr_iterator_range (iter, &range_start, &range_end);
+	  have_next = pango_attr_iterator_next (&iter);
+	  pango_attr_iterator_range (&iter, &range_start, &range_end);
 
 	  if (range_start >= state.iter.end_index) /* New range doesn't intersect this cluster */
 	    {
@@ -680,7 +697,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
 	    }
 
 	  state.segment_attrs = g_slist_concat (state.segment_attrs,
-						pango_attr_iterator_get_attrs (iter));
+						pango_attr_iterator_get_attrs (&iter));
 	}
       while (have_next);
     }
@@ -694,14 +711,14 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
   if (LTR (glyph_item))
     result = g_slist_reverse (result);
 
-  pango_attr_iterator_destroy (iter);
+  _pango_attr_iterator_destroy (&iter);
 
   return result;
 }
 
 /**
  * pango_glyph_item_letter_space:
- * @glyph_item: a #PangoGlyphItem
+ * @glyph_item: a `PangoGlyphItem`
  * @text: text that @glyph_item corresponds to
  *   (glyph_item->item->offset is an offset from the
  *    start of @text)
@@ -716,7 +733,7 @@ pango_glyph_item_apply_attrs (PangoGlyphItem   *glyph_item,
  * give the effect of typographic letter spacing.
  *
  * Since: 1.6
- **/
+ */
 void
 pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
 			       const char     *text,
@@ -774,7 +791,7 @@ pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
 
 /**
  * pango_glyph_item_get_logical_widths:
- * @glyph_item: a #PangoGlyphItem
+ * @glyph_item: a `PangoGlyphItem`
  * @text: text that @glyph_item corresponds to
  *   (glyph_item->item->offset is an offset from the
  *    start of @text)
@@ -783,12 +800,13 @@ pango_glyph_item_letter_space (PangoGlyphItem *glyph_item,
  *                  glyph_item->item->num_chars) to be filled in with
  *                  the resulting character widths.
  *
- * Given a #PangoGlyphItem and the corresponding
- * text, determine the screen width corresponding to each character. When
- * multiple characters compose a single cluster, the width of the entire
+ * Given a `PangoGlyphItem` and the corresponding text, determine the width
+ * corresponding to each character.
+ *
+ * When multiple characters compose a single cluster, the width of the entire
  * cluster is divided equally among the characters.
  *
- * See also pango_glyph_string_get_logical_widths().
+ * See also [method@Pango.GlyphString.get_logical_widths].
  *
  * Since: 1.26
  **/
