@@ -233,6 +233,7 @@ do_test (const gchar *filename,
   if (g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
     {
       g_test_skip ("Test file not found");
+      g_error_free (error);
       return;
     }
 
@@ -298,6 +299,7 @@ do_test (const gchar *filename,
         }
       g_free (string);
       g_free (expected_attrs);
+      g_free (line);
 
       i++;
     }
